package net.darkmist.defcon18;

import java.util.Arrays;

import junit.framework.TestCase;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class UtilTest extends TestCase
{
	private static final Class<UtilTest> CLASS = UtilTest.class;
	@SuppressWarnings("unused")
	private static final Logger logger = LoggerFactory.getLogger(CLASS);

	private static byte[] getAllBytes()
	{
		byte[] bytes = new byte[256];
		for(int b=0;b<256;b++)
			bytes[b]=(byte)b;
		return bytes;
	}

	public void testToHexString()
	{
		byte[] bytes = new byte[1];
		String actual;
		String expected;

		for(int b=Byte.MIN_VALUE;b<=Byte.MAX_VALUE;b++)
		{
			if((expected = Integer.toHexString(b&0xFF)).length()<2)
				expected = "0" + expected;
			bytes[0]=(byte)b;
			actual = Util.toHexString(bytes);
			assertEquals(expected, actual);
		}
	}

	public void testEncodeDecode()
	{
		byte[] input = getAllBytes();
		byte[] expected = getAllBytes();
		byte[] actual;

		actual = Util.decode(Util.encode(input));
		assertTrue(Arrays.equals(expected,actual));
	}

	public void testGetResourceString()
	{
		String expected = "toast is yummy\n(like cabbage)\n";
		String actual;

		actual = Util.getResourceString(CLASS, "toast");
		assertEquals(expected,actual);
	}

	public void testGetResourceStringRef()
	{
		String expected = "toast is yummy\n(like cabbage)\n";
		CharSequence actual;

		actual = Util.getResourceStringRef(CLASS, "toast");
		assertEquals(expected,actual.toString());
	}

	public void testIfNullNotNull()
	{
		String input = "toast is yummy";
		String expected = input;
		String actual;

		actual = Util.ifNull(input);
		assertEquals(expected, actual);
	}

	public void testIfNullNull()
	{
		String input = null;
		String expected = "(null)";
		String actual;

		actual = Util.ifNull(input);
		assertEquals(expected, actual);
	}

	public void testJoin()
	{
		Object[] input = new Object[]{"a",Long.valueOf(1),null,"b"};
		String sep = "X";
		String expected = "aX1X(null)Xb";
		String actual;

		actual = Util.join(sep, input);
		assertEquals(expected,actual);
	}
}
